package raytracer.scenes;

//Copyright (C) Helen Hu 2011.
//based on C++ code from Ray Tracing from the Ground Up, by Kevin Suffern 
//This Java code is for non-commercial purposes only.
//This Java code is licensed under the GNU General Public License Version 2.
//See the file COPYING.txt for the full license.

/**
 * This class contains the code to creating scenes from chapter 3.
 */

import raytracer.world.World;
import raytracer.geometricObjects.*;
import raytracer.tracers.*;
import raytracer.utilities.*;

public class Chapter3 {

	// Listing 3.11 on page 68
	// creates scene shown in Figure 3.18
	public static void buildSingleSphere(World w) {
		w.background_color = RGBColor.white;
		w.tracer = new SingleSphere(w); 

		w.sphere.set_center(new Point3D(0,0,0));
		w.sphere.set_radius(85);
	}

	// Listing 3.18 on page 77
	// creates scene shown in Figure 3.20
	public static void buildMultipleObjects(World w) {
		w.tracer = new MultipleObjects(w); 

		w.background_color = RGBColor.black;

		// use access functions to set centre and radius

		Sphere sphere = new Sphere();
		sphere.set_center(0, -25, 0);
		sphere.set_radius(80);
		sphere.set_color(1, 0, 0);  // red
		w.add_object(sphere);

		// use constructor to set centre and radius 

		sphere = new Sphere(new Point3D(0, 30, 0), 60);
		sphere.set_color(1, 1, 0);	// yellow
		w.add_object(sphere);

		Plane plane_ptr = new Plane(new Point3D(0), new Normal(0, 1, 1));
		plane_ptr.set_color(0.0f, 0.3f, 0.0f);	// dark green
		w.add_object(plane_ptr);
	}

	// This builds the cover image for Chapter 3: Bare Bones Ray Tracing (page 45)
	public static void buildBBCoverPic(World w) {
		w.tracer = new MultipleObjects(w);  

		// spheres

		Sphere	sphere1 = new Sphere(new Point3D(5, 3, 0), 30); 
		sphere1.set_color(RGBColor.yellow);	   								// yellow
		w.add_object(sphere1);

		Sphere	sphere2 = new Sphere(new Point3D(45, -7, -60), 20); 
		sphere2.set_color(RGBColor.brown);									// brown
		w.add_object(sphere2);

		Sphere	sphere3 = new Sphere(new Point3D(40, 43, -100), 17); 
		sphere3.set_color(RGBColor.dark_green);								// dark green
		w.add_object(sphere3);

		Sphere	sphere4 = new Sphere(new Point3D(-20, 28, -15), 20); 
		sphere4.set_color(RGBColor.orange);									// orange
		w.add_object(sphere4);

		Sphere	sphere5 = new Sphere(new Point3D(-25, -7, -35), 27); 			
		sphere5.set_color(RGBColor.green);									// green
		w.add_object(sphere5);

		Sphere	sphere6 = new Sphere(new Point3D(20, -27, -35), 25); 
		sphere6.set_color(RGBColor.light_green);								// light green
		w.add_object(sphere6);

		Sphere	sphere7 = new Sphere(new Point3D(35, 18, -35), 22); 
		sphere7.set_color(RGBColor.green);   								// green
		w.add_object(sphere7);

		Sphere	sphere8 = new Sphere(new Point3D(-57, -17, -50), 15);  
		sphere8.set_color(RGBColor.brown);									// brown
		w.add_object(sphere8);

		Sphere	sphere9 = new Sphere(new Point3D(-47, 16, -80), 23); 
		sphere9.set_color(RGBColor.light_green);							// light green
		w.add_object(sphere9);

		Sphere	sphere10 = new Sphere(new Point3D(-15, -32, -60), 22); 
		sphere10.set_color(RGBColor.dark_green);     						// dark green
		w.add_object(sphere10);

		Sphere	sphere11 = new Sphere(new Point3D(-35, -37, -80), 22); 
		sphere11.set_color(RGBColor.dark_yellow);							// dark yellow
		w.add_object(sphere11);

		Sphere	sphere12 = new Sphere(new Point3D(10, 43, -80), 22); 
		sphere12.set_color(RGBColor.dark_yellow);							// dark yellow
		w.add_object(sphere12);

		Sphere	sphere13 = new Sphere(new Point3D(30, -7, -80), 10); 
		sphere13.set_color(RGBColor.dark_yellow);
		w.add_object(sphere13);												// dark yellow (hidden)

		Sphere	sphere14 = new Sphere(new Point3D(-40, 48, -110), 18); 
		sphere14.set_color(RGBColor.dark_green); 							// dark green
		w.add_object(sphere14);

		Sphere	sphere15 = new Sphere(new Point3D(-10, 53, -120), 18); 
		sphere15.set_color(RGBColor.brown); 								// brown
		w.add_object(sphere15);

		Sphere	sphere16 = new Sphere(new Point3D(-55, -52, -100), 10); 
		sphere16.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere16);

		Sphere	sphere17 = new Sphere(new Point3D(5, -52, -100), 15); 		
		sphere17.set_color(RGBColor.brown);									// browm
		w.add_object(sphere17);

		Sphere	sphere18 = new Sphere(new Point3D(-20, -57, -120), 15); 
		sphere18.set_color(RGBColor.dark_purple);							// dark purple
		w.add_object(sphere18);

		Sphere	sphere19 = new Sphere(new Point3D(55, -27, -100), 17); 
		sphere19.set_color(RGBColor.dark_green);							// dark green
		w.add_object(sphere19);

		Sphere	sphere20 = new Sphere(new Point3D(50, -47, -120), 15); 
		sphere20.set_color(RGBColor.brown);									// browm
		w.add_object(sphere20);

		Sphere	sphere21 = new Sphere(new Point3D(70, -42, -150), 10); 
		sphere21.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere21);

		Sphere	sphere22 = new Sphere(new Point3D(5, 73, -130), 12); 
		sphere22.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere22);

		Sphere	sphere23 = new Sphere(new Point3D(66, 21, -130), 13); 			
		sphere23.set_color(RGBColor.dark_purple);							// dark purple
		w.add_object(sphere23);	

		Sphere	sphere24 = new Sphere(new Point3D(72, -12, -140), 12); 
		sphere24.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere24);

		Sphere	sphere25 = new Sphere(new Point3D(64, 5, -160), 11); 			
		sphere25.set_color(RGBColor.green);					 				// green
		w.add_object(sphere25);

		Sphere	sphere26 = new Sphere(new Point3D(55, 38, -160), 12); 		
		sphere26.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere26);

		Sphere	sphere27 = new Sphere(new Point3D(-73, -2, -160), 12); 		
		sphere27.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere27);

		Sphere	sphere28 = new Sphere(new Point3D(30, -62, -140), 15); 
		sphere28.set_color(RGBColor.dark_purple); 							// dark purple
		w.add_object(sphere28);

		Sphere	sphere29 = new Sphere(new Point3D(25, 63, -140), 15); 
		sphere29.set_color(RGBColor.dark_purple);							// dark purple
		w.add_object(sphere29);

		Sphere	sphere30 = new Sphere(new Point3D(-60, 46, -140), 15);  
		sphere30.set_color(RGBColor.dark_purple); 							// dark purple
		w.add_object(sphere30);

		Sphere	sphere31 = new Sphere(new Point3D(-30, 68, -130), 12); 
		sphere31.set_color(RGBColor.light_purple); 							// light purple
		w.add_object(sphere31);

		Sphere	sphere32 = new Sphere(new Point3D(58, 56, -180), 11);   
		sphere32.set_color(RGBColor.green);									//  green
		w.add_object(sphere32);

		Sphere	sphere33 = new Sphere(new Point3D(-63, -39, -180), 11); 
		sphere33.set_color(RGBColor.green);									// green 
		w.add_object(sphere33);

		Sphere	sphere34 = new Sphere(new Point3D(46, 68, -200), 10); 	
		sphere34.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere34);

		Sphere	sphere35 = new Sphere(new Point3D(-3, -72, -130), 12); 
		sphere35.set_color(RGBColor.light_purple);							// light purple
		w.add_object(sphere35);
	}


}
